/*
 * stream_phys.java
 * Douglas S. J. De Couto
 * 31 August 1999
 */

/*
 * Copyright (C) 1999 Douglas S. J. De Couto
 * <decouto@lcs.mit.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package decouto.garble;

import java.io.*;

/** A stream-based implementation of the phsical layer interface,
    which can be used by the link layer.  This implementation does not
    support timeouts; i.e. none are ever generated.  This
    implementation can be used to hook up a serial port to the garble
    library.
    @author Douglas S. J. De Couto <a href="mailto:decouto@lcs.mit.edu">decouto@lcs.mit.edu</a>
    @version 31 August 1999 0.1 
    @see decouto.garble.physical_layer
    @see javax.comm.SerialPort */
class stream_phys implements physical_layer {
  private InputStream m_is;
  private OutputStream m_os;
  private boolean m_init = false;

  private FileOutputStream m_of;
  private FileOutputStream m_if;

  /** Creates an unuseable physical layer.  You should not use this
      constructor. */
  stream_phys() {
    m_is = null;
    m_os = null;
    m_init = false;
  }

  /** Creates a physical layer which outputs bytes to os, and reads
      bytes from is.
      @param is InputStream to read bytes from.
      @param os OutputStream to write bytes to.
      @see java.io.InputStream
      @see java.io.OutputStream */
  stream_phys(InputStream is, OutputStream os) {
    if (is == null || os == null) {
      m_init = false;
      m_os = null;
      m_is = null;
    }
    else {
      m_is = is; 
      m_os = os;
      m_init = true;
      
      try { // setup mirror stream logs
      	m_of = new FileOutputStream("phys_layer_out", true);
      	m_if = new FileOutputStream("phys_layer_in", true);
      }
      catch (IOException e) {
      	System.out.println("physical_layer: can't open mirror file streams");
      	m_of = null;
      	m_if = null;
      }
      try {
      	String sep = "----- new session -----\n";
      	m_of.write(sep.getBytes());
      	m_if.write(sep.getBytes());
      }
      catch (IOException e) { }
    }
  }

  public void putbyte(byte b) throws not_possible, timeout {
    if (!m_init) {
      throw new not_possible("Not initialized");
    }
    byte b_arr[] = { b };
    try {
      m_os.write(b_arr);
      String s = Integer.toHexString(b & 0xff);
      s += "\n";
      m_of.write(s.getBytes());
    }
    catch (IOException e) { }
  }

  public byte getbyte() throws not_possible, timeout {
    if (!m_init) {
      throw new not_possible("Not initialized");
    }
    int i = 0;
    try {
      i = m_is.read();
      if (i == -1) {
	throw new not_possible("End of input stream");
      }
      String s = Integer.toHexString(i) + "\n";
      m_if.write(s.getBytes());
    }
    catch (IOException e) {
      throw new not_possible(e.getMessage());
    }
    
    return (byte) i;
  }
} 

